//-----------------------------------------------------------------------------
//  Copyright (C) 2008 Thomas S. Ullrich 
//
//  This file is part of "eDISK".
//
//  This file may be used under the terms of the GNU General Public License.
//  This project is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License.
//  
//  Author: Thomas S. Ullrich
//  Last update: March 28, 2008
//-----------------------------------------------------------------------------
#include <QtGui>
#include <QtDebug>
#include <cmath>
#include <vector>
#include <float.h>
#include <iostream>
#include "eDISKWindow.h"
#include "eDISKinematics.h"
#include "eDISKHelpBrowser.h"

using namespace std;

eDISKWindow::eDISKWindow()
{
    //
    //  Init members
    //
    mRadianUnits = false;
    
    //
    //  Create widgets, docks, and dialogs
    //
    mImageView = new QGraphicsView;
    mImageScene = new QGraphicsScene(mImageView);
    mImageView->setScene(mImageScene);
    mImageView->setCacheMode(QGraphicsView::CacheBackground);
    mImageView->setViewportUpdateMode(QGraphicsView::FullViewportUpdate);
    setCentralWidget(mImageView);
    mImageScene->installEventFilter(this);
    mImageScene->setBackgroundBrush(Qt::white);    
    mImageView->setRenderHint (QPainter::Antialiasing, true );
    mImageView->setRenderHint (QPainter::HighQualityAntialiasing, true );
    mImageView->setRenderHint (QPainter::TextAntialiasing, true );
    
    mViewport = QRectF(QPointF(-100, 100), QPointF(100, -100));
    createGraphicsScene();
        
    createMenuActions();
    createMenus();
    createStatusBar();
    createDockingWindow();

    //
    //  Set main window properties
    //
    setWindowTitle(tr("eDISK"));
    setWindowIcon(QIcon(QString::fromUtf8(":/images/eDISKLogo128.png")));
    QSize s = size();
    s.setWidth(1000);
    resize(s);
    ensurePolished();
    
    //
    //  Connect signals and slots
    //
    connect(mModeSelectionComboBox, SIGNAL(currentIndexChanged (int)), this, SLOT(setMode(int)));
    connectAll();
    
    //
    //  Make the help browser
    //
    createHelpBrowser();
    
    //
    //  Init kinematics
    //
    mEngine = 0;
    setMode(mXY);
    recalculateAndUpdate();
}

void eDISKWindow::about() 
{
    QDate date(QDate::currentDate());
    char title[64];
    char text[2024];
    sprintf(title, "eDISK");
    sprintf(text, "eDISK Version %.2f (%s %d)\n\n"            
            "Copyright (C) %d Thomas Ullrich (BNL)\n\n"
            "This program is free software; you can redistribute it and/or "
            "modify it under the terms of the GNU General Public License as "
            "published by the Free Software Foundation; either version 2 of "
            "the License, or (at your option) any later version.\n\n"
            "This program is distributed  in the hope that it will be useful, "
            "but without any warranty; without even the implied warranty of "
            "merchantability or fitness for a particular purpose. See the GNU "
            "General Public License for more details.\n", VERSION, 
            QDate::longMonthName(date.month()).toAscii().constData(), date.year(), 
            date.year());
    QMessageBox::about(this, title, text);
}

void eDISKWindow::closeEvent(QCloseEvent*)
{
    //
    //  Reimplement, otherwise pressing the x button on the
    //  window frame closes the applications and we have
    //  no chance for storing the settings and checking
    //  for unsaved data.
    //
    finish();
}

void eDISKWindow::connectAll() 
{
    connect(mField_Ebeam_e, SIGNAL(valueChanged(double)), this, SLOT(recalculateAndUpdate()));
    connect(mField_Ebeam_p, SIGNAL(valueChanged(double)), this, SLOT(recalculateAndUpdate()));
    connect(mField_x, SIGNAL(returnPressed()), this, SLOT(recalculateAndUpdate()));
    connect(mField_y, SIGNAL(valueChanged(double)), this, SLOT(recalculateAndUpdate()));
    connect(mField_Q2, SIGNAL(valueChanged(double)), this, SLOT(recalculateAndUpdate()));
    connect(mField_Ee, SIGNAL(returnPressed()), this, SLOT(recalculateAndUpdate()));
    connect(mField_ae, SIGNAL(valueChanged(double)), this, SLOT(recalculateAndUpdate()));
    connect(mField_Eq, SIGNAL(returnPressed()), this, SLOT(recalculateAndUpdate()));
    connect(mField_aq, SIGNAL(valueChanged(double)), this, SLOT(recalculateAndUpdate()));
}

void eDISKWindow::createDockingWindow()
{    
    mKinematicsDock = new QDockWidget(tr("Kinematics Window"), this);
    mKinematicsDock->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea | Qt::BottomDockWidgetArea | Qt::TopDockWidgetArea);
    QFrame *form = new QFrame(mKinematicsDock);
    createDockingWindowElements(form);
    mKinematicsDock->setWidget(form);        
    addDockWidget(Qt::RightDockWidgetArea, mKinematicsDock);
    mViewMenu->addAction(mKinematicsDock->toggleViewAction());
}

void eDISKWindow::createDockingWindowElements(QWidget *form)
{
    int row = 0;
    QVBoxLayout *vboxLayout = new QVBoxLayout(form);
    
    //
    //   Beams and C.M. Energy Group Box
    //
    QGroupBox *beamGroupBox = new QGroupBox(form);
    QGridLayout *gridLayout1 = new QGridLayout(beamGroupBox);
    beamGroupBox->setTitle(tr("Beam Parameters:"));   
    
    // Electron Beam (GeV)
    QLabel *label_1 = new QLabel(beamGroupBox);
    label_1->setTextFormat(Qt::RichText);
    label_1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
    label_1->setText(QApplication::translate("gridLayout1", "Electron Beam E (GeV):", 0, QApplication::UnicodeUTF8));
    gridLayout1->addWidget(label_1, row, 0);
    
    mField_Ebeam_e = new QDoubleSpinBox(beamGroupBox);
    mField_Ebeam_e->setAccelerated(true);
    mField_Ebeam_e->setMinimum(1.);
    mField_Ebeam_e->setMaximum(1.0e6);
    mField_Ebeam_e->setSingleStep(1.0);
    mField_Ebeam_e->setValue(10.);
    gridLayout1->addWidget(mField_Ebeam_e, row, 1); 
    
    QSpacerItem *spacerItemA = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);    
    gridLayout1->addItem(spacerItemA, row, 2);    
    row++;
    
    
    // Proton Beam (GeV)
    QLabel *label_2 = new QLabel(beamGroupBox);
    label_2->setTextFormat(Qt::RichText);
    label_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
    label_2->setText(QApplication::translate("form", "Proton Beam E (GeV):", 0, QApplication::UnicodeUTF8));
    gridLayout1->addWidget(label_2, row, 0);
    
    mField_Ebeam_p = new QDoubleSpinBox(beamGroupBox);
    mField_Ebeam_p->setAccelerated(true);
    mField_Ebeam_p->setMinimum(1.);
    mField_Ebeam_p->setMaximum(1.0e6);
    mField_Ebeam_p->setSingleStep(5.0);
    mField_Ebeam_p->setValue(100.);
    gridLayout1->addWidget(mField_Ebeam_p, row, 1);

    QSpacerItem *spacerItemB = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);    
    gridLayout1->addItem(spacerItemB, row, 2);
    row++;
        
    // Sqrt(s) GeV
    QLabel *label_3 = new QLabel(beamGroupBox);
    label_3->setTextFormat(Qt::RichText);
    label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
    label_3->setText(QApplication::translate("Form", "\342\210\232s (GeV):", 0, QApplication::UnicodeUTF8));
    gridLayout1->addWidget(label_3, row, 0);
    
    mField_sqrtS = new QLineEdit(beamGroupBox);
    mField_sqrtS->setReadOnly(true);
    gridLayout1->addWidget(mField_sqrtS, row, 1);
    QSpacerItem *spacerItemC = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);    
    gridLayout1->addItem(spacerItemC, row, 2);
    row++;
    
    vboxLayout->addWidget(beamGroupBox);

    //
    //  DIS kinematics box
    //
    QGroupBox *disGroupBox = new QGroupBox(form);
    QGridLayout *gridLayout2 = new QGridLayout(disGroupBox);
    disGroupBox->setTitle(tr("DIS Kinematics"));   
    row = 0;
    
    // Mode selector
    QLabel *modeLabel = new QLabel(disGroupBox);
    modeLabel->setTextFormat(Qt::RichText);
    modeLabel->setAlignment(Qt::AlignLeft|Qt::AlignTrailing|Qt::AlignVCenter);
    modeLabel->setText("Based on:");
    mModeSelectionComboBox = new QComboBox(disGroupBox);
    mModeSelectionComboBox->clear();
    mModeSelectionComboBox->insertItems(0, QStringList()
                                        << tr("x and y")
                                        << tr("x and Q\262")
                                        << tr("y and Q\262")
                                        << tr("Scattered Electron")
                                        << tr("Scattered Quark")
                                        );
    mModeSelectionComboBox->setCurrentIndex(0);
    gridLayout2->addWidget(modeLabel, row, 0);
    gridLayout2->addWidget(mModeSelectionComboBox, row, 1, 1, 1);
    row++;

    // separator
    QFrame *line = new QFrame(disGroupBox);
    line->setFrameShape(QFrame::HLine);
    line->setFrameShadow(QFrame::Sunken);    
    gridLayout2->addWidget(line, row, 0, 1, 4);
    row++;
    
    // x  
    QLabel *label_5 = new QLabel(disGroupBox);
    label_5->setTextFormat(Qt::RichText);
    label_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
    label_5->setText("x:");
    gridLayout2->addWidget(label_5, row, 0);
    
    mField_x = new QLineEdit(disGroupBox);
    mField_x->setText("1e-2");
    gridLayout2->addWidget(mField_x, row, 1);
    
    // y  
    QLabel *label_4 = new QLabel(disGroupBox);
    label_4->setTextFormat(Qt::RichText);
    label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
    label_4->setText("y:");
    gridLayout2->addWidget(label_4, row, 2);
    
    mField_y = new QDoubleSpinBox(disGroupBox);
    mField_y->setAccelerated(true);
    mField_y->setMinimum(0.0);
    mField_y->setMaximum(1.0);
    mField_y->setSingleStep(0.02);
    mField_y->setValue(0.9);
    mField_y->setDecimals(4);
    gridLayout2->addWidget(mField_y, row, 3);
    row++;
        
    // Q2  
    QLabel *label_6 = new QLabel(disGroupBox);
    label_6->setTextFormat(Qt::RichText);
    label_6->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
    label_6->setText("Q\262 (GeV\262):");
    gridLayout2->addWidget(label_6, row, 0);
    
    mField_Q2 = new QDoubleSpinBox(disGroupBox);
    mField_Q2->setAccelerated(true);
    mField_Q2->setMinimum(0.0);
    mField_Q2->setMaximum(1e6);
    mField_Q2->setDecimals(3);
    mField_Q2->setSingleStep(0.25);
    mField_Q2->setValue(1);
    gridLayout2->addWidget(mField_Q2, row, 1);
    row++;
    
    // separator
    QFrame *line0 = new QFrame(disGroupBox);
    line0->setFrameShape(QFrame::HLine);
    line0->setFrameShadow(QFrame::Sunken);    
    gridLayout2->addWidget(line0, row, 0, 1, 4);
    row++;
    
    // Scattered Electron 
    QLabel *label_7 = new QLabel(disGroupBox);
    label_7->setTextFormat(Qt::RichText);
    label_7->setAlignment(Qt::AlignLeft|Qt::AlignVCenter);
    label_7->setText("Scattered Electron:");
    gridLayout2->addWidget(label_7, row, 0, 1, 2);
    row++;

    QLabel *label_8 = new QLabel(disGroupBox);
    label_8->setTextFormat(Qt::RichText);
    label_8->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
    label_8->setText("E (GeV):");
    gridLayout2->addWidget(label_8, row, 0);
    
    mField_Ee = new QLineEdit(disGroupBox);
    gridLayout2->addWidget(mField_Ee, row, 1);
    
    mLabelElectronAngle = new QLabel(disGroupBox);
    mLabelElectronAngle->setTextFormat(Qt::RichText);
    mLabelElectronAngle->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
    if (mRadianUnits)
        mLabelElectronAngle->setText(QApplication::translate("Form", "\316\270 (rad)", 0, QApplication::UnicodeUTF8));
    else
        mLabelElectronAngle->setText(QApplication::translate("Form", "\316\270 (deg)", 0, QApplication::UnicodeUTF8));
    gridLayout2->addWidget(mLabelElectronAngle, row, 2);
    
    mField_ae = new QDoubleSpinBox(disGroupBox);
    mField_ae->setAccelerated(true);
    mField_ae->setMinimum(0.0);
    mField_ae->setMaximum(180);
    mField_ae->setDecimals(4);
    mField_ae->setSingleStep(1);
    gridLayout2->addWidget(mField_ae, row, 3);
    row++;

    // separator
    QFrame *line1 = new QFrame(disGroupBox);
    line1->setFrameShape(QFrame::HLine);
    line1->setFrameShadow(QFrame::Sunken);    
    gridLayout2->addWidget(line1, row, 0, 1, 4);
    row++;
        
    // Scatted quark  
    QLabel *label_10 = new QLabel(disGroupBox);
    label_10->setTextFormat(Qt::RichText);
    label_10->setAlignment(Qt::AlignLeft|Qt::AlignVCenter);
    label_10->setText("Scattered Quark:");
    gridLayout2->addWidget(label_10, row, 0, 1, 2);
    row++;
    
    QLabel *label_11 = new QLabel(disGroupBox);
    label_11->setTextFormat(Qt::RichText);
    label_11->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
    label_11->setText("E (GeV):");
    gridLayout2->addWidget(label_11, row, 0);
    
    mField_Eq = new QLineEdit(disGroupBox);
    gridLayout2->addWidget(mField_Eq, row, 1);

    mLabelQuarkAngle = new QLabel(disGroupBox);
    mLabelQuarkAngle->setTextFormat(Qt::RichText);
    mLabelQuarkAngle->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
    if (mRadianUnits)
        mLabelQuarkAngle->setText(QApplication::translate("Form", "\316\263 (rad)", 0, QApplication::UnicodeUTF8));
    else
        mLabelQuarkAngle->setText(QApplication::translate("Form", "\316\263 (deg)", 0, QApplication::UnicodeUTF8));
    gridLayout2->addWidget(mLabelQuarkAngle, row, 2);
    
    mField_aq = new QDoubleSpinBox(disGroupBox);
    mField_aq->setAccelerated(true);
    mField_aq->setMinimum(0.0);
    mField_aq->setMaximum(180);
    mField_aq->setDecimals(4);
    mField_aq->setSingleStep(1);
    gridLayout2->addWidget(mField_aq, row, 3);
    row++;

    // separator
    QFrame *line2 = new QFrame(disGroupBox);
    line2->setFrameShape(QFrame::HLine);
    line2->setFrameShadow(QFrame::Sunken);    
    gridLayout2->addWidget(line2, row, 0, 1, 4);
    row++;

    // nu and W2
    QLabel *label_13 = new QLabel(disGroupBox);
    label_13->setTextFormat(Qt::RichText);
    label_13->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
    label_13->setText(QApplication::translate("Form", "\316\275 (GeV)", 0, QApplication::UnicodeUTF8));
    gridLayout2->addWidget(label_13, row, 0);
    
    mField_nu = new QLineEdit(disGroupBox);
    mField_nu->setReadOnly(true);
    gridLayout2->addWidget(mField_nu, row, 1);
    
    QLabel *label_14 = new QLabel(disGroupBox);
    label_14->setTextFormat(Qt::RichText);
    label_14->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
    label_14->setText("W\262 (GeV\262):");
    gridLayout2->addWidget(label_14, row, 2);
    
    mField_W2 = new QLineEdit(disGroupBox);
    mField_W2->setReadOnly(true);
    gridLayout2->addWidget(mField_W2, row, 3);
    row++;
    
    // end dis group
    vboxLayout->addWidget(disGroupBox);
    
    // last row
    QSpacerItem *spacerItem_last = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);
    vboxLayout->addItem(spacerItem_last);
    
}

void eDISKWindow::createGraphicsScene()
{
    //
    // Coordinate system
    //
    // x direction:
    // mViewport.x() = -100
    // mViewport.width() = 200
    // mViewport.left() = -100
    // mViewport.right() = 100
    //
    // y direction
    // mViewport.y() = 100
    // mViewport.height() = -200
    // mViewport.top() = 100  (is actually bottom)
    // mViewport.bottom() = -100  (is actually top)
    //
    //
    // (-100,-100)          (100,-100)
    //
    //
    //              (0,0)
    //
    //
    // (-100,100)           (100, 100)
    //
    //
    
    QFont sansFont("Helvetica", 10);
    
    //
    //  Axis
    //
    mBeamline = new QGraphicsLineItem(mViewport.left(), 0, mViewport.right(), 0); 
    mBeamline->setZValue(2);
    mBeamline->setPen(QPen(Qt::black));
    mBeamline->setVisible(true);
    mImageScene->addItem(mBeamline);
    
    mVerticalAxis = new QGraphicsLineItem(0, mViewport.top(), 0, mViewport.bottom()); 
    mVerticalAxis->setZValue(2);
    mVerticalAxis->setPen(QPen(Qt::black));
    mVerticalAxis->setVisible(true);
    mImageScene->addItem(mVerticalAxis);
    
    //
    //  Indicate incoming e and p
    //
    double y = -3;
    double x = mViewport.x()+mViewport.width()/40;
    double len = mViewport.width()/5;
    double aheight = len/10;
    double alen = len/4;
    mElectronDirectionLine = new QGraphicsLineItem(x, y, x+len, y); 
    mElectronDirectionLine->setZValue(2);
    mElectronDirectionLine->setPen(QPen(Qt::red));
    mElectronDirectionLine->setVisible(true);
    mImageScene->addItem(mElectronDirectionLine);
    QPolygonF poly1;
    poly1.push_back(QPointF(x+len, y));
    poly1.push_back(QPointF(x+len-alen, y+aheight/2.));
    poly1.push_back(QPointF(x+len-alen, y-aheight/2.));
    mElectronDirectionArrow = new QGraphicsPolygonItem(poly1);
    mElectronDirectionArrow->setZValue(2);
    mElectronDirectionArrow->setPen(QPen(Qt::red));
    mElectronDirectionArrow->setBrush(QBrush(Qt::red));
    mElectronDirectionArrow->setVisible(true);
    mImageScene->addItem(mElectronDirectionArrow);
    
    mElectronDirectionText = new QGraphicsTextItem();
    mElectronDirectionText->setHtml("<i>e</i>");
    mElectronDirectionText->setDefaultTextColor(Qt::red);
    mElectronDirectionText->setPos(x+len/4, y);
    mElectronDirectionText->setFont(sansFont);
    mElectronDirectionText->setVisible(true);
    mImageScene->addItem(mElectronDirectionText);
    
    x = mViewport.x()+mViewport.width()-mViewport.width()/40;
    mProtonDirectionLine = new QGraphicsLineItem(x, y, x-len, y); 
    mProtonDirectionLine->setZValue(2);
    mProtonDirectionLine->setPen(QPen(Qt::blue));
    mProtonDirectionLine->setVisible(true);
    mImageScene->addItem(mProtonDirectionLine);
    QPolygonF poly2;
    poly2.push_back(QPointF(x-len, y));
    poly2.push_back(QPointF(x-len+alen, y+aheight/2.));
    poly2.push_back(QPointF(x-len+alen, y-aheight/2.));
    mProtonDirectionArrow = new QGraphicsPolygonItem(poly2);
    mProtonDirectionArrow->setZValue(2);
    mProtonDirectionArrow->setPen(QPen(Qt::blue));
    mProtonDirectionArrow->setBrush(QBrush(Qt::blue));
    mProtonDirectionArrow->setVisible(true);
    mImageScene->addItem(mProtonDirectionArrow);
    
    mProtonDirectionText = new QGraphicsTextItem();
    mProtonDirectionText->setHtml("<i>p</i>");
    mProtonDirectionText->setDefaultTextColor(Qt::blue);
    mProtonDirectionText->setPos(x-len/2, y);
    mProtonDirectionText->setFont(sansFont);
    mProtonDirectionText->setVisible(true);
    mImageScene->addItem(mProtonDirectionText);
    
    //
    //  Degree circle
    //
    double r = 50; 
    double dr = 4; 
    mAngleIndicator = new QGraphicsEllipseItem (QRectF(-r, r, 2*r, -2*r));
    mAngleIndicator->setPen(QPen(Qt::black));
    mAngleIndicator->setVisible(true);
    mImageScene->addItem(mAngleIndicator);
    QGraphicsLineItem *item;
    const double torad = M_PI/180.;
    double x1, x2, y1, y2, theta, tlen;
    for (int i=0; i<360; i+=5) {
        tlen = i%10 ? dr/2 : dr; // long tics for 10th, short for 5th
        theta = i*torad;
        x1 = r*cos(theta);
        y1 = -r*sin(theta);
        x2 = (r-tlen)*cos(theta);
        y2 = -(r-tlen)*sin(theta);
        item = new QGraphicsLineItem(x1, y1, x2, y2);
        item->setPen(QPen(Qt::black));
        item->setVisible(true);
        mImageScene->addItem(item);
        mAngleIndicatorTics.push_back(item); 
    }
    //
    //  Create the outgoing lines but do not draw them yet.
    //
    mScatteredElectronLine = new QGraphicsLineItem();
    QPen pen1(Qt::red); pen1.setWidthF(0.5);
    mScatteredElectronLine->setPen(pen1);
    mScatteredElectronLine->setVisible(false);
    mScatteredElectronLine->setZValue(2);
    mImageScene->addItem(mScatteredElectronLine);
    
    mScatteredQuarkLine = new QGraphicsLineItem();
    QPen pen2(Qt::darkBlue); pen2.setWidthF(0.5);
    mScatteredQuarkLine->setPen(pen2);
    mScatteredQuarkLine->setVisible(false);
    mScatteredQuarkLine->setZValue(2);
    mImageScene->addItem(mScatteredQuarkLine);
}

void eDISKWindow::createHelpBrowser()
{
    mHelpBrowser = 0;
    
    QString path = qApp->applicationDirPath() + "/docs";
    QDir dir(path);
    if (!dir.exists()) {
        path = qApp->applicationDirPath() + "/../docs";
        dir.setPath(path);
    }
#if defined(Q_OS_MAC)   
    if (!dir.exists()) {
        path = qApp->applicationDirPath() + "/../Resources/docs";
        dir.setPath(path);
    }
#endif
    //
    //  For developing on Mac/Xcode only: avoids having to copy
    //  the docs into the build substructure.
    //
    if (!dir.exists()) { 
        path = "/Users/ullrich/Documents/Projects/eDISK/docs";
        dir.setPath(path);
    }
    if (!dir.exists()) {
        QMessageBox::warning( 0, "eDISK",
                              "Cannot find the directory holding the documentation (docs). "
                              "No online help will be available. "
                              "Check your installation and reinstall if necessary.");
        return;
    }
    
    QString descFile = path + "/doc.index";
    
    QFileInfo file(descFile);
    if (file.exists())
        mHelpBrowser = new eDISKHelpBrowser(descFile, path);
    else {
        QMessageBox::warning( 0, "eDISK",
                              "Cannot find the index descriptor to setup the documentation. "
                              "No online help will be available. "
                              "Check your installation and reinstall if necessary.");
    }
}

void eDISKWindow::createMenus()
{
    mFileMenu = menuBar()->addMenu(tr("&File"));
    mFileMenu->addAction(mPrintAction);
#if !defined(Q_WS_MAC)
    mFileMenu->addSeparator();
#endif
    mFileMenu->addAction(mFinishAction);

    mViewMenu = menuBar()->addMenu(tr("&View"));
    mViewMenu->addAction(mViewRadiansAction);
    // dock windows toggle menues added later via createDockWindows()
    
    mHelpMenu = menuBar()->addMenu(tr("&Help"));
    mHelpMenu->addAction(mHelpAction);
#if !defined(Q_WS_MAC)
    mHelpMenu->addSeparator();
#endif    
    mHelpMenu->addAction(mAboutAction);  // In application menu on MacOS/X
}

void eDISKWindow::createMenuActions()
{    
    mPrintAction = new QAction(tr("&Print..."), this);
    mPrintAction->setShortcut(tr("Ctrl+P"));
    mPrintAction->setCheckable(false);
    connect(mPrintAction, SIGNAL(triggered()), this, SLOT(print()));

    mViewRadiansAction = new QAction(tr("Show &Radians"), this);
    mViewRadiansAction->setShortcut(tr("Ctrl+R"));
    mViewRadiansAction->setCheckable(true);
    mViewRadiansAction->setChecked(mRadianUnits);
    connect(mViewRadiansAction, SIGNAL(triggered()), this, SLOT(setAngularUnits()));
    
    mFinishAction = new QAction(tr("&Quit eDISK"), this);
    mFinishAction->setShortcut(tr("Ctrl+Q"));
    connect(mFinishAction, SIGNAL(triggered()), this, SLOT(finish()));
    
    mAboutAction = new QAction(tr("&About eDISK"), this);
    mAboutAction->setCheckable(false);
    connect(mAboutAction, SIGNAL(triggered()), this, SLOT(about()));
    
    mHelpAction = new QAction(tr("&Contents"), this);
    mHelpAction->setCheckable(false);
    mHelpAction->setShortcut(tr("F1"));
    connect(mHelpAction, SIGNAL(triggered()), this, SLOT(help()));
}

void eDISKWindow::createStatusBar()
{
    statusBar()->showMessage(tr("Ready"));
}

void eDISKWindow::disconnectAll() 
{
    disconnect(mField_Ebeam_e, SIGNAL(valueChanged(double)), this, SLOT(recalculateAndUpdate()));
    disconnect(mField_Ebeam_p, SIGNAL(valueChanged(double)), this, SLOT(recalculateAndUpdate()));
    disconnect(mField_x, SIGNAL(returnPressed()), this, SLOT(recalculateAndUpdate()));
    disconnect(mField_y, SIGNAL(valueChanged(double)), this, SLOT(recalculateAndUpdate()));
    disconnect(mField_Q2, SIGNAL(valueChanged(double)), this, SLOT(recalculateAndUpdate()));
    disconnect(mField_Ee, SIGNAL(returnPressed()), this, SLOT(recalculateAndUpdate()));
    disconnect(mField_ae, SIGNAL(valueChanged(double)), this, SLOT(recalculateAndUpdate()));
    disconnect(mField_Eq, SIGNAL(returnPressed()), this, SLOT(recalculateAndUpdate()));
    disconnect(mField_aq, SIGNAL(valueChanged(double)), this, SLOT(recalculateAndUpdate()));
}

void eDISKWindow::drawScene()
{   
    //
    //  For drawing coordinate frame see comments in
    //  createGraphicsScene()
    //
    double x, y;
    
    // 
    //  Check for errors
    //
    if (mEngine->error()) {
        mScatteredElectronLine->setVisible(false);
        mScatteredQuarkLine->setVisible(false);
        fitInView();
        return;
    }
    
    //
    //  Define the scale first
    //
    double Emax = mEngine->electronE() > mEngine->quarkE() ? mEngine->electronE() : mEngine->quarkE();
    double scale = 100/Emax;  // scene-units/GeV
    
    //
    //  Electron
    //
    x = mEngine->electronE()*scale*cos(M_PI-mEngine->theta());
    y = mEngine->electronE()*scale*sin(M_PI-mEngine->theta());
    mScatteredElectronLine->setLine(0, 0, x, -y);
    mScatteredElectronLine->setVisible(true);
    mScatteredElectronTip = QPointF(x, -y);
    
    //
    //  Quark
    //
    x = mEngine->quarkE()*scale*cos(M_PI-mEngine->gamma());
    y = mEngine->quarkE()*scale*sin(M_PI-mEngine->gamma());
    mScatteredQuarkLine->setLine(0, 0, x, y);
    mScatteredQuarkLine->setVisible(true);
    mScatteredQuarkTip = QPointF(x, y);
    
    fitInView();
}

bool eDISKWindow::event(QEvent* e)
{
    if (e->type() == QEvent::PolishRequest || e->type() == QEvent::Resize) 
    {
        fitInView();
    }
    return false;
}

bool eDISKWindow::eventFilter(QObject *obj, QEvent *event)
{
    //
    // If method retuns true  the event stops here otherwise its
    // passed down. Here we catch all dispatched events from the
    // image scene. Later we can here allow playing with the mouse.
    // 
    QGraphicsSceneMouseEvent *mevt;
    static bool leftMousePressedNearElectronTip = false;
    static bool leftMousePressedNearQuarkTip = false;
    QString num;
    double radius2;

    if (obj == mImageScene) {
        
        // needed for rescaling the quark or electron energy
        double Emax = mEngine->electronE() > mEngine->quarkE() ? mEngine->electronE() : mEngine->quarkE();
        double scale = 100/Emax;  // scene-units/GeV
        
        if (event->type() == QEvent::GraphicsSceneMousePress) {
            mevt = dynamic_cast<QGraphicsSceneMouseEvent*>(event);  

            QPointF mpos = mevt->scenePos();
            QPointF vedist = mpos - mScatteredElectronTip;
            double edist = vedist.x()*vedist.x()+vedist.y()*vedist.y();
            QPointF vqdist = mpos - mScatteredQuarkTip;
            double qdist = vqdist.x()*vqdist.x()+vqdist.y()*vqdist.y();
            const float distmax = 25;
            if (edist < distmax || qdist < distmax) {
                leftMousePressedNearElectronTip = (edist < qdist);
                leftMousePressedNearQuarkTip = (qdist < edist);
            }
            else {
                leftMousePressedNearElectronTip = false;
                leftMousePressedNearQuarkTip = false;
            }
            if (leftMousePressedNearElectronTip) mModeSelectionComboBox->setCurrentIndex(mElectron);
            if (leftMousePressedNearQuarkTip) mModeSelectionComboBox->setCurrentIndex(mQuark);
            return true;
        }
        else if (event->type() == QEvent::GraphicsSceneMouseRelease) {
            leftMousePressedNearElectronTip = false;
            leftMousePressedNearQuarkTip = false;
            return true;
        }
        else if (event->type() == QEvent::GraphicsSceneMouseMove && leftMousePressedNearElectronTip) {
            mevt = dynamic_cast<QGraphicsSceneMouseEvent*>(event);    
            QPointF epos = mevt->scenePos();
            double theta = ( M_PI - atan2(fabs(epos.y()),epos.x()));
            if (mRadianUnits)
                mField_ae->setValue(theta);      
            else
                mField_ae->setValue(theta*180/M_PI);      
            // recalculateAndUpdate();  not needed   
            radius2 =  epos.y()*epos.y() + epos.x()*epos.x();
            if (radius2 > 10000) radius2 = 10000;
            num.setNum(mEngine->electronE()*sqrt(radius2)/(mEngine->electronE()*scale)); 
            mField_Ee->setText(num);  
            return true;
        }
        else if (event->type() == QEvent::GraphicsSceneMouseMove && leftMousePressedNearQuarkTip) {
            mevt = dynamic_cast<QGraphicsSceneMouseEvent*>(event);    
            QPointF qpos = mevt->scenePos();
            double gamma = ( M_PI - atan2(fabs(qpos.y()),qpos.x()));
            if (mRadianUnits)
                mField_aq->setValue(gamma);      
            else
                mField_aq->setValue(gamma*180/M_PI);      
            radius2 =  qpos.y()*qpos.y() + qpos.x()*qpos.x();
            if (radius2 > 10000) radius2 = 10000;
            num.setNum(mEngine->quarkE()*sqrt(radius2)/(mEngine->quarkE()*scale)); 
            mField_Eq->setText(num);  
            return true;
        }
        else
            return true;
    }
    return QObject::eventFilter(obj, event);  // standard event processing
}

void eDISKWindow::finish() 
{
    QApplication::exit(0);
}

void eDISKWindow::fitInView()
{
    mImageView->centerOn(0, 0);
    mImageView->fitInView (mViewport, Qt::KeepAspectRatio);
    update();
}

void eDISKWindow::help() 
{
    if (mHelpBrowser) 
        mHelpBrowser->show(); 
    else
        QMessageBox::warning( 0, "eDISK",
                              "Sorry no help available.\n"
                              "Documentation files are missing in this installation. "
                              "Check your installation and reinstall if necessary.");    
}

void eDISKWindow::print() 
{
    //
    //  Create document for printing
    //
    QTextDocument document;
    QTextCursor cursor(&document);
    
    QTextCharFormat textFormat;
    textFormat.setFontPointSize(14);
    QTextCharFormat boldFormat;
    boldFormat.setFontWeight(QFont::Bold);
    
    cursor.movePosition(QTextCursor::Start);
    
    //
    //  Header
    //
    cursor.insertText(tr("eDISK Version %1\n").arg(VERSIONSTRING), textFormat);
    cursor.insertBlock();   
    
    //
    //  Image
    //
    QImage image(QSize(400,400), QImage::Format_RGB32);
    QPainter painter(&image);
    painter.setRenderHint(QPainter::Antialiasing, true );
    painter.setRenderHint(QPainter::HighQualityAntialiasing, true );
    painter.setRenderHint(QPainter::TextAntialiasing, true );
    mImageScene->render(&painter);
    document.addResource(QTextDocument::ImageResource, QUrl("image"), image);
    cursor.insertImage("image");   
    cursor.insertBlock();
    
    //
    //  Kinematic variables
    //
    cursor.insertText("Beams:\n", textFormat);
    cursor.insertText(tr("\tE(e) = %1 GeV\n").arg(mField_Ebeam_e->value()), textFormat);
    cursor.insertText(tr("\tE(p) = %1 GeV\n").arg(mField_Ebeam_p->value()), textFormat);
    cursor.insertText(QApplication::translate("Form","\t\342\210\232s = %1 GeV\n", 0, QApplication::UnicodeUTF8).arg(mField_sqrtS->text()), textFormat);
    cursor.insertText("DIS:\n", textFormat);
    cursor.insertText(tr("\tx = %1 \n").arg(mField_x->text()), textFormat);
    cursor.insertText(tr("\ty = %1 \n").arg(mField_y->value()), textFormat);
    cursor.insertText(tr("\tQ\262 = %1 GeV\262\n").arg(mField_Q2->value()), textFormat);
    cursor.insertText(tr("\tE'(e) = %1 GeV\n").arg(mField_Ee->text()), textFormat);
    if (mRadianUnits)
        cursor.insertText(QApplication::translate("Form", "\t\316\270 = %1 rad\n", 0, QApplication::UnicodeUTF8).arg(mField_ae->value()), textFormat);
    else
        cursor.insertText(QApplication::translate("Form", "\t\316\270 = %1 deg\n", 0, QApplication::UnicodeUTF8).arg(mField_ae->value()), textFormat);
    cursor.insertText(tr("\tE(q) = %1 GeV\n").arg(mField_Eq->text()), textFormat);
    if (mRadianUnits)
        cursor.insertText(QApplication::translate("Form", "\t\316\263 = %1 rad\n", 0, QApplication::UnicodeUTF8).arg(mField_aq->value()), textFormat);
    else
        cursor.insertText(QApplication::translate("Form", "\t\316\263 = %1 deg\n", 0, QApplication::UnicodeUTF8).arg(mField_aq->value()), textFormat);
    cursor.insertText(tr("\tW\262 = %1 GeV\262\n").arg(mField_W2->text()), textFormat);
    cursor.insertText(QApplication::translate("Form", "\t\316\275 = %1 GeV\n", 0, QApplication::UnicodeUTF8).arg(mField_nu->text()), textFormat);

    //
    //  Start printer dialog and print the document we just created
    //
    QPrinter printer(QPrinter::HighResolution);
    
    QPrintDialog *dlg = new QPrintDialog(&printer, this);
    dlg->setWindowTitle(tr("eDISK - Print"));
    if (dlg->exec() != QDialog::Accepted)
        return;
    
    document.print(&printer);
    
    statusBar()->showMessage(tr("Ready"));
}

void eDISKWindow::recalculateAndUpdate()
{
    const double torad = M_PI/180.;
    const double todeg = 180./M_PI;
    QString num;
    
    //
    // Disconnect all signals to avoid signals being emitted
    // when changing fileds programatically.
    //
    disconnectAll();
    
    //
    //  Get beam energies and create the DIS kinematic engine
    //
    double Ee = mField_Ebeam_e->value();
    double Ep = mField_Ebeam_p->value();
    if (mEngine) delete mEngine;
    mEngine = new eDISKinematics(Ee, Ep);
    
    //
    //  Show sqrt(s) ans set kinematic limits (if possible)
    //
    num.setNum(mEngine->sqrts());
    mField_sqrtS->setText(num);
    // mField_Ebeam_p->setMaximum(mEngine->maxQ2());
    
    //
    //  Calculate all kinematic variables depending on input
    //
    bool OK = true;
    int mode = mModeSelectionComboBox->currentIndex();
    switch (mode) {
        case mXY:
            OK = mEngine->setxy(mField_x->text().toDouble(), mField_y->value());
            break;
        case mXQ2:
            OK = mEngine->setxQ2(mField_x->text().toDouble(), mField_Q2->value());
            break;
        case mYQ2:
            OK = mEngine->setyQ2(mField_y->text().toDouble(), mField_Q2->value());
            break;
        case mElectron:
            if (mRadianUnits)
                OK = mEngine->setScatteredElectron(mField_Ee->text().toDouble(), mField_ae->value());
            else
                OK = mEngine->setScatteredElectron(mField_Ee->text().toDouble(), mField_ae->value()*torad);                
            break;
        case mQuark:
            if (mRadianUnits)
                OK = mEngine->setScatteredQuark(mField_Eq->text().toDouble(), mField_aq->value());
            else
                OK = mEngine->setScatteredQuark(mField_Eq->text().toDouble(), mField_aq->value()*torad);
            break;
        default:
            break;
    }
    
    char text[256];
    if (OK) {
        statusBar()->showMessage(tr("Calculation OK"));
    }
    else {
        sprintf(text, "Error calculating kinematics, error id = %d.", mEngine->error());
        statusBar()->showMessage(text);
    }
    
    
    //
    //  Update all fields with new values.
    //  Don't update the input ones.
    //
    char c;
    if (mode != mXY && mode != mXQ2) {
        c = mEngine->x() < 0.1 ? 'e' : 'f'; 
        num.setNum(mEngine->x(), c); 
        mField_x->setText(num);
    }        
    if (mode != mXY && mode != mYQ2) mField_y->setValue(mEngine->y());  
    
    if (mode != mYQ2 && mode != mXQ2) mField_Q2->setValue(mEngine->Q2());        
    
    if (mode != mElectron) {
        if (mRadianUnits)
            mField_ae->setValue(mEngine->theta());      
        else
            mField_ae->setValue(mEngine->theta()*todeg);      
        num.setNum(mEngine->electronE()); 
        mField_Ee->setText(num);  
    }
    
    if (mode != mQuark) {
        num.setNum(mEngine->quarkE()); 
        mField_Eq->setText(num);  
        if (mRadianUnits)
            mField_aq->setValue(mEngine->gamma()); 
        else
            mField_aq->setValue(mEngine->gamma()*todeg); 
    }
    
    num.setNum(mEngine->nu()); mField_nu->setText(num);  
    num.setNum(mEngine->W2()); mField_W2->setText(num);  
    
    //
    //  All done update and reconnect signals, draw result
    //
    drawScene();
    connectAll();
}

void eDISKWindow::setAngularUnits()
{
    const double torad = M_PI/180.;
    const double todeg = 180./M_PI;

    bool old = mRadianUnits;
    if (mViewRadiansAction->isChecked()) {
        mRadianUnits = true;
        mLabelElectronAngle->setText(QApplication::translate("Form", "\316\270 (rad)", 0, QApplication::UnicodeUTF8));
        mLabelQuarkAngle->setText(QApplication::translate("Form", "\316\263 (rad)", 0, QApplication::UnicodeUTF8));   
        if (old != mRadianUnits) {
            mField_ae->setValue(mField_ae->value()*torad); 
            mField_aq->setValue(mField_aq->value()*torad); 
        }
    }
    else {
        mRadianUnits = false;
        mLabelElectronAngle->setText(QApplication::translate("Form", "\316\270 (deg)", 0, QApplication::UnicodeUTF8));
        mLabelQuarkAngle->setText(QApplication::translate("Form", "\316\263 (deg)", 0, QApplication::UnicodeUTF8));
        if (old != mRadianUnits) {
            mField_ae->setValue(mField_ae->value()*todeg); 
            mField_aq->setValue(mField_aq->value()*todeg); 
        }
    }    
    update();
}

void eDISKWindow::setMode(int mode) 
{
    QPalette palette, palette1;
    
	palette = QPalette( mField_Q2->palette() );
    if (mode == mXQ2 || mode == mYQ2) {
        mField_Q2->setReadOnly(false);
        palette.setColor( QPalette::Text, Qt::blue );
        mField_Q2->setPalette(palette);
    }
    else {
        mField_Q2->setReadOnly(true);
        palette.setColor( QPalette::Text, Qt::black );
        mField_Q2->setPalette(palette);
    }
    
    palette = QPalette( mField_x->palette() );
    if (mode == mXY || mode == mXQ2) {
        mField_x->setReadOnly(false);
        palette.setColor( QPalette::Text, Qt::blue );
        mField_x->setPalette(palette);
    }
    else {
        mField_x->setReadOnly(true);
        palette.setColor( QPalette::Text, Qt::black );
        mField_x->setPalette(palette);
    }
    
    palette = QPalette( mField_y->palette() );
    if (mode == mXY || mode == mYQ2) {
        mField_y->setReadOnly(false);
        palette.setColor( QPalette::Text, Qt::blue );
        mField_y->setPalette(palette);
    }
    else {
        mField_y->setReadOnly(true);
        palette.setColor( QPalette::Text, Qt::black );
        mField_y->setPalette(palette);
    }
    
    palette = QPalette( mField_Ee->palette() );
    palette1 = QPalette( mField_ae->palette() );
    if (mode == mElectron) {
        mField_Ee->setReadOnly(false);
        mField_ae->setReadOnly(false);
        palette.setColor( QPalette::Text, Qt::blue );
        palette1.setColor( QPalette::Text, Qt::blue );
        mField_Ee->setPalette(palette);
        mField_ae->setPalette(palette1);
    }
    else {
        mField_Ee->setReadOnly(true);
        mField_ae->setReadOnly(true);
        palette.setColor( QPalette::Text, Qt::black );
        palette1.setColor( QPalette::Text, Qt::black );
        mField_Ee->setPalette(palette);
        mField_ae->setPalette(palette1);
    }
    
    palette = QPalette( mField_Eq->palette() );
    palette1 = QPalette( mField_aq->palette() );
    if (mode == mQuark) {
        mField_Eq->setReadOnly(false);
        mField_aq->setReadOnly(false);
        palette.setColor( QPalette::Text, Qt::blue );
        palette1.setColor( QPalette::Text, Qt::blue );
        mField_Eq->setPalette(palette);
        mField_aq->setPalette(palette1);
    }
    else {
        mField_Eq->setReadOnly(true);
        mField_aq->setReadOnly(true);
        palette.setColor( QPalette::Text, Qt::black );
        palette1.setColor( QPalette::Text, Qt::black );
        mField_Eq->setPalette(palette);
        mField_aq->setPalette(palette1);
    }   
    update();
}


