//-----------------------------------------------------------------------------
//  Copyright (C) 2008 Thomas S. Ullrich 
//
//  This file is part of "eDISK".
//
//  This file may be used under the terms of the GNU General Public License.
//  This project is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License.
//  
//  Author: Thomas S. Ullrich
//  Last update: Feb 10, 2008
//-----------------------------------------------------------------------------
#include "eDISKinematics.h"
#include <cmath>
#include <iostream>
#define PR(x) cout << #x << " = " << (x) << endl;

using namespace std;

const double eDISKinematics::mProtonMass = 0.938272;  
const double eDISKinematics::mElectronMass = 0.510998918e-3;

eDISKinematics::eDISKinematics(double Ee, double Ep) 
{
    mElectronBeamE = Ee;
    mProtonBeamE = Ep;
    double pe = sqrt(mElectronBeamE*mElectronBeamE-mElectronMass*mElectronMass);
    double pp = -sqrt(mProtonBeamE*mProtonBeamE-mProtonMass*mProtonMass);
    mS = mElectronMass*mElectronMass + mProtonMass*mProtonMass + 2*mElectronBeamE*mProtonBeamE - 2*pe*pp;

    mX = mY = mQ2 = -1;   
    mScatteredElectronE = mScatteredElectronAngle = -1;
    mScatteredQuarkE = mScatteredQuarkAngle = -1;
    mNu = mW2 = -1;
    
    // to speed up calculations
    mM2m2 = mElectronMass*mElectronMass + mProtonMass*mProtonMass;
    
    mError = 0;
}

bool eDISKinematics::setxy(double x, double y) 
{
    mError = 0;
    if (x < 0 || x > 1) {mError = 1; return false;}
    mX = x;
    if (y < 0 || y > 1) {mError = 2; return false;}
    mY = y;
    
    mQ2 = (mS-mM2m2)*mX*mY;
    
    return calculateScatteredElectron() && 
        calculateScatteredQuark() &&
        calculateFinalStateHadrons();
}

bool eDISKinematics::setxQ2(double x, double q2) 
{
    mError = 0;
    if (x < 0 || x > 1) {mError = 3; return false;}
    mX = x;

    if (q2 > mS-mM2m2 || q2 < 0) {mError = 4; return false;}
    mQ2 = q2;

    mY = mQ2/(mX*(mS-mM2m2));
    if (isnan(mY)) {mError = 20; return false;}

    
    return calculateScatteredElectron() && 
        calculateScatteredQuark() &&
        calculateFinalStateHadrons();
}

bool eDISKinematics::setyQ2(double y, double q2) 
{
    mError = 0;
    if (y < 0 || y > 1) {mError = 5; return false;}
    mY = y;
    
    if (q2 > mS-mM2m2 || q2 < 0) {mError = 6; return false;}
    mQ2 = q2;
    
    mX = mQ2/(mY*(mS-mM2m2));
    if (isnan(mX)) {mError = 21; return false;}

    return calculateScatteredElectron() && 
        calculateScatteredQuark() &&
        calculateFinalStateHadrons();
}

bool eDISKinematics::setScatteredElectron(double E, double theta)
{
    mError = 0;
    if (theta < 0 || theta > M_PI || E < 0) {mError = 7; return false;}
    
    double costheta = cos(theta);
    double q2 = 2*mElectronBeamE*E*(1+costheta);
    if (q2 > mS-mM2m2 || q2 < 0) {mError = 8; return false;}
    
    double y = 1 - E*(1-costheta)/(2*mElectronBeamE);
    if (y < 0 || y > 1) {mError = 9; return false;}
    
    double x = q2/(y*(mS-mM2m2));
    if (x < 0 || x > 1) {mError = 10; return false;}
    
    mQ2 = q2;
    mX = x;
    mY = y;
    mScatteredElectronAngle = theta;
    mScatteredElectronE = E;

    return calculateScatteredQuark() &&
        calculateFinalStateHadrons();
}

bool eDISKinematics::setScatteredQuark(double E, double gamma)
{
    mError = 0;
    if (gamma  < 0 || gamma  > M_PI || E  < 0) {mError = 11; return false;};
    
    double ghalf = gamma/2.;
    double y     =  E*(sin(ghalf)*sin(ghalf))/mElectronBeamE;
    if (y < 0 || y > 1) {mError = 12; return false;};
    
    double q2   = (E*sin(gamma)*E*sin(gamma))/(1-y);
    if (q2 > mS-mM2m2 || q2 < 0) {mError = 13; return false;};
    
    double x = q2/(y*(mS-mM2m2));
    if (x < 0 || x > 1) {mError = 14; return false;};
    
    mQ2 = q2;
    mX = x;
    mY = y;
    mScatteredQuarkAngle = gamma;
    mScatteredQuarkE = E;
    
    return calculateScatteredElectron() &&
        calculateFinalStateHadrons();
}

bool eDISKinematics::calculateScatteredElectron()
{
    //
    // Define energy and angle of scattered electron given y and Q2
    //
    
    mScatteredElectronE  = mQ2/(4.*mElectronBeamE) + mElectronBeamE*(1-mY);
    double costheta= mQ2/(2.*mScatteredElectronE*mElectronBeamE) - 1;
    if(costheta < -1 || costheta > 1) {mError = 15; return false;};
    mScatteredElectronAngle = acos(costheta);

    if (isnan(mScatteredElectronE)) {mError = 17; return false;}
    return true;
}

bool eDISKinematics::calculateScatteredQuark()
{
    //
    // Define energy and angles of scattered quark given y and Q2
    //
    
    mScatteredQuarkE  = mY*mElectronBeamE + mQ2*(1-mY)/(4*mElectronBeamE*mY);
    double cosgamma= 1 - 2*mY*mElectronBeamE/(mY*mElectronBeamE + mQ2*(1-mY)/(4*mElectronBeamE*mY));
    if(cosgamma < -1 || cosgamma > 1) {mError = 16; return false;};
    mScatteredQuarkAngle = acos(cosgamma);
    
    if (isnan(mScatteredQuarkE)) {mError = 18; return false;}
    
    return true;
}

bool eDISKinematics::calculateFinalStateHadrons()
{
    //
    // Define final state hadron variables given x and Q2
    //
    mNu  = mQ2/(2*mX *mProtonMass);
    mW2 = mQ2*(1-mX)/mX + mProtonMass*mProtonMass;
    if (isnan(mW2)) {mError = 19; return false;}
    
    return true;
}

double eDISKinematics::maxQ2() const
{
    return mS-mM2m2;
}

//
//  Access functions 
//
double eDISKinematics::s() const {return mS;}
double eDISKinematics::sqrts() const {return sqrt(mS);}
double eDISKinematics::electronBeamE() const {return mElectronBeamE;}
double eDISKinematics::protonBeamE() const {return mProtonBeamE;}
double eDISKinematics::Q2() const {return mQ2;}
double eDISKinematics::x() const {return mX;}
double eDISKinematics::y() const {return mY;}
double eDISKinematics::theta() const {return mScatteredElectronAngle;}
double eDISKinematics::gamma() const {return mScatteredQuarkAngle;}
double eDISKinematics::quarkE() const {return mScatteredQuarkE;}
double eDISKinematics::electronE() const {return mScatteredElectronE;}
double eDISKinematics::W2() const {return mW2;}
double eDISKinematics::nu() const {return mNu;}
int    eDISKinematics::error() const {return mError;}
