//-----------------------------------------------------------------------------
//  Copyright (C) 2008 Thomas S. Ullrich 
//
//  This file is part of "eDISK".
//
//  This file may be used under the terms of the GNU General Public License.
//  This project is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License.
//  
//  Author: Thomas S. Ullrich
//  Last update: Feb 10, 2008
//-----------------------------------------------------------------------------
//
//   Instantiated via:
//       eDISKinematic(electron_beam_energy, proton_beam_energy);
//   This calculates the CM energy right away.
//
//   The whole event kinematics is defined through 2 variables. 
//   There are several combinations and all are covered. Once
//   one of them is invoked the whole DIS event kinematics is
//   defined:
//       bool setxy(double x, double y);
//       bool setxQ2(double x, double Q2);
//       bool setyQ2(double y, double Q2);
//       bool setScatteredElectron(double E, double theta);
//       bool setScatteredQuark(double E, double gamma);
//
//   Note that theta and gamma are the angles with respect to the incoming
//   electron. This is a common source of confusion. 
//   If all kinematic calculations work out and are in range each of these
//   methods returns true. If anything went wrong false. In this case the
//   latest error can be retrieved with:
//   int error();
//
//   You need to check in the sopurce code what the error means. The class
//   itself does not produce any error messages.
//
//   Once everything is defined all kinematic variables can be accessed via
//   the usual access functions.
//
//   All units are in GeV (or GeV2) and radians.
//-----------------------------------------------------------------------------
#ifndef eDISKinematics_H
#define eDISKinematics_H

class eDISKinematics {
public:
    eDISKinematics(double, double);
    
    double s() const;
    double sqrts() const;
    double electronBeamE() const;
    double protonBeamE() const;
    double Q2() const;
    double x() const;
    double y() const;
    double theta() const;
    double gamma() const;
    double quarkE() const;
    double electronE() const;
    double W2() const;
    double nu() const;
    
    double maxQ2() const;
    int    error() const;
    
    bool setxy(double, double);
    bool setxQ2(double, double);
    bool setyQ2(double, double);
    bool setScatteredElectron(double, double);
    bool setScatteredQuark(double, double);
    
private:
    bool calculateScatteredQuark();
    bool calculateScatteredElectron();
    bool calculateFinalStateHadrons();
    
private:
    static const double mElectronMass;
    static const double mProtonMass;
    double mM2m2;
    double mElectronBeamE;
    double mProtonBeamE;
    double mS;
    double mX;
    double mY;
    double mQ2;
    double mScatteredElectronE;
    double mScatteredElectronAngle;
    double mScatteredQuarkE;
    double mScatteredQuarkAngle;
    double mNu;
    double mW2;
    int    mError;
};

#endif
